@@ -5,12 +5,13 @@
 #
 #  Distributed under the terms of the Artistic License 2.0
 #
-#  (c) David Griffith 2012-2015
+#  (c) David Griffith 2012-2019
 #
 #  Latest version is at https://gitlab.com/DavidGriffith/blorbtools
 # ---------------------------------------------------------------------------
 
 use strict;
+use DateTime;
 use File::Temp qw(tempfile tempdir);
 use Encode qw(encode);
 use Getopt::Long;
@@ -20,10 +21,16 @@ my $file_sep	= '/';		# Character used to separate directories in
 				# pathnames (on most systems this will be /)
 
 my $blurb_filename  = 'input.blurb';
-my $output_filename = 'output.blb';
-my $version = "perlBlorb 3.0, part of Blorbtools";
+my $output_blorb = 'output.blb';
+my $output_inform = 'manifest.inf';
+my $output_zil = 'manifest.zil';
+my $version = "pblorb 3.1";
 my $temp_dir = tempdir(CLEANUP => 1);
 
+my $output_filename;
+
+my $create_blorb = 0;
+
 my $blurb_line = 0;
 my $chunk_count = 0;
 my $chunk_opened = 0;
@@ -64,11 +71,16 @@ my @picture_numbering;
 my @sound_numbering;
 
 my %options;
-GetOptions('usage|?'	=> \$options{usage},
-	'h|help'	=> \$options{help},
-	'n|nobuild'	=> \$options{nobuild}
+my $getret;
+
+$getret = GetOptions('usage|?'	=> \$options{usage},
+	'h|help'		=> \$options{help},
+	'i|inform'		=> \$options{inform},
+	'o|output=s'		=> \$options{output},
+	'z|zil'			=> \$options{zil}
 	);
 
+if (!$getret) { pod2usage(1); }
 
 # ---------------------------------------------------------------------------
 
@@ -76,33 +88,64 @@ GetOptions('usage|?'	=> \$options{usage},
 
 sub error
 {   my $m = $_[0];
-    print STDERR "$blurb_filename, line $blurb_line: Error: $m\n";
+    print STDERR "** $blurb_filename, line $blurb_line: Error: $m\n";
 }
 
 sub fatal
 {   my $m = $_[0];
-    die "$blurb_filename, line $blurb_line: Fatal error: $m\n";
+    die "** $blurb_filename, line $blurb_line: Fatal error: $m\n";
 }
 
 # ---------------------------------------------------------------------------
 
+sub print_banner
+{
+    my $string = $_[0];
+
+    # We're creating a Blorb file.
+    if ($create_blorb) {
+	print "  $string\n";
+	return;
+    }
+
+    # We're creating a manifest file.
+    if ($options{inform}) { print OUTFILE "!"; }
+    elsif ($options{zil}) { print OUTFILE "\""; }
+    else { die "** Unknown manifest type\n" }
+
+    if ($output_filename) { print "  $string\n"; }
+    print OUTFILE "  $string";
+
+    if ($options{inform}) { print OUTFILE "\n"; }
+    elsif ($options{zil}) { print OUTFILE "  \"\n"; }
+    else { die "** Unknown manifest type\n" }
+}
+
+
+
+sub write_chunk
+{   my $n = $_[0];
+    print CHUNK $n;
+}
+
+
 sub four_word
 {   my $n = $_[0];
-    print CHUNK sprintf("%c%c%c%c", ($n / 0x1000000),
+    write_chunk(sprintf("%c%c%c%c", ($n / 0x1000000),
                                     ($n / 0x10000)%0x100,
                                     ($n / 0x100)%0x100,
-                                    ($n)%0x100);
+                                    ($n)%0x100));
 }
 
 sub two_word
 {   my $n = $_[0];
-    print CHUNK sprintf("%c%c", ($n / 0x100),
-                                  ($n)%0x100);
+    write_chunk(sprintf("%c%c", ($n / 0x100),
+                                  ($n)%0x100));
 }
 
 sub one_byte
 {   my $n = $_[0];
-    print CHUNK sprintf("%c", $n);
+    write_chunk(sprintf("%c", $n));
 }
 
 sub begin_chunk
@@ -111,6 +154,9 @@ sub begin_chunk
     my $chunk_filename = $_[2];
     $chunk_opened = 0;
 
+    # If we're building a manifest, just return.
+    if ($options{inform} || $options{zil}) { return; }
+
     if ($cnum > $max_resource_num) { $max_resource_num = $cnum; }
 
     if ($chunk_filename eq "")
@@ -125,12 +171,7 @@ sub begin_chunk
     $chunk_filename_array[$chunk_count] = $chunk_filename;
 
     $chunk_important_array[$chunk_count] = 0;
-    if (($id eq "PNG ") || ($id eq "JPEG") || ($id eq "GIF ")
-	|| ($id eq "GFX ")
-	|| ($id eq "AIFF") || ($id eq "MOD ") || ($id eq "OGGV")
-	|| ($id eq "WAVE") || ($id eq "MIDI") || ($id eq "MP3 ")
-	|| ($id eq "ZCOD") || ($id eq "GLUL") || ($id eq "MAGS")
-	|| ($id eq "ADRI"))
+    if (isgraphics($id) || isaudio($id) || isexec($id))
     {   $chunk_important_array[$chunk_count] = 1;
         $important_count = $important_count + 1;
     }
@@ -143,6 +184,9 @@ sub begin_chunk
 sub end_chunk
 {   my ($size, $blen, $buffer);
 
+    # If we're building a manifest, just return.
+    if ($options{inform} || $options{zil}) { return; }
+
     if ($chunk_opened == 1)
     {   close(CHUNK);
     }
@@ -150,7 +194,7 @@ sub end_chunk
     $chunk_filename = $chunk_filename_array[$chunk_count];
 
     open(CHUNK, $chunk_filename)
-        or fatal("unable to open $chunk_filename for size counting");
+        or fatal("unable to open $chunk_filename");
     binmode(CHUNK);
 
     for ($size = 0, $blen = 1; $blen > 0; )
@@ -179,14 +223,14 @@ sub end_chunk
 sub author_chunk
 {   my $t = $_[0];
     begin_chunk("AUTH", 0, "");
-    print CHUNK $t;
+    write_chunk($t);
     end_chunk();
 }
 
 sub copyright_chunk
 {   my $t = $_[0];
     begin_chunk("(c) ", 0, "");
-    print CHUNK $t;
+    write_chunk($t);
     end_chunk();
 }
 
@@ -216,12 +260,66 @@ sub storyname_chunk
 {   my $t = $_[0];
     my $foo = encode("UTF16-BE", $t);
     begin_chunk("SNam", 0, "");
-    print CHUNK $foo;
+    write_chunk($foo);
     end_chunk();
 }
 
+
 # ---------------------------------------------------------------------------
 
+
+# Check if this chunk contains audio data.
+#
+sub isaudio
+{
+    my $id = $_[0];
+
+    if ($id eq "AIFF") { return 1; }
+    if ($id eq "MOD ") { return 1; }
+    if ($id eq "OGGV") { return 1; }
+    if ($id eq "WAVE") { return 1; }
+    if ($id eq "MIDI") { return 1; }
+    if ($id eq "MP3 ") { return 1; }
+    return 0;
+}
+
+
+# Check if this chunk contains graphics data.
+#
+sub isgraphics
+{
+    my $id = $_[0];
+
+    if ($id eq "PNG ") { return 1; }
+    if ($id eq "JPEG") { return 1; }
+    if ($id eq "GIF ") { return 1; }
+    if ($id eq "GFX ") { return 1; }
+    return 0;
+}
+
+
+# Check if this chunk is an executable.
+#
+sub isexec
+{
+    my $id = $_[0];
+
+    if ($id eq "ZCOD") { return 1; }
+    if ($id eq "GLUL") { return 1; }
+    if ($id eq "TAD2") { return 1; }
+    if ($id eq "TAD3") { return 1; }
+    if ($id eq "HUGO") { return 1; }
+    if ($id eq "ALAN") { return 1; }
+    if ($id eq "ADRI") { return 1; }
+    if ($id eq "LEVE") { return 1; }
+    if ($id eq "AGT ") { return 1; }
+    if ($id eq "MAGS") { return 1; }
+    if ($id eq "ADVS") { return 1; }
+    if ($id eq "EXEC") { return 1; }
+    return 0;
+}
+
+
 # The mod file formats listed here are the ones supported by libmodplug,
 # which is the mod player library used in Unix Frotz and Windows Frotz.
 #
@@ -255,11 +353,18 @@ sub ismod
 }
 
 
-
 # ---------------------------------------------------------------------------
 
+
 sub identify
-{   print STDOUT "Constant $_[0] = $_[1];\n";
+{
+    if ($options{inform}) { print OUTFILE "Constant $_[0] = $_[1];\n"; return; }
+    if ($options{zil}) {
+	my $constant = uc $_[0];
+	print OUTFILE "<CONSTANT $constant $_[1]>\n";
+	return;
+    }
+
 }
 
 sub interpret
@@ -358,10 +463,20 @@ sub interpret
 	    begin_chunk("ZCOD", 0, $filename);
 	} elsif ($ext eq "ulx") {
 	    begin_chunk("GLUL", 0, $filename);
-	} elsif ($ext eq "taf") {
-	    begin_chunk("ADRI", 0, $filename);
+	} elsif ($ext eq "gam") {
+	    begin_chunk("TAD2", 0, $filename);
+	} elsif ($ext eq "t3") {
+	    begin_chunk("TAD3", 0, $filename);
+	} elsif ($ext eq "hex") {
+	    begin_chunk("HUGO", 0, $filename);
+	} elsif ($ext eq "dat") {
+	    begin_chunk("LEVE", 0, $filename);
 	} elsif ($ext eq "mag") {
 	    begin_chunk("MAGS", 0, $filename);
+	} elsif ($ext eq "dat") {
+	    begin_chunk("ADVS", 0, $filename);
+	} elsif ($ext eq "taf") {
+	    begin_chunk("ADRI", 0, $filename);
 	} else {
 	    fatal("unknown executable extension $ext");
 	}
@@ -375,8 +490,12 @@ sub interpret
         binmode(IDFILE);
         begin_chunk("IFhd", 0, "");
         $version = unpack("C", getc(IDFILE));
-        print STDOUT "! Identifying v$version story file $1\n";
 
+	if ($options{inform}) {
+            print OUTFILE "! Identifying v$version story file $1\n";
+        } elsif ($options{zil}) {
+            print OUTFILE "\" Identifying v$version story file $1\"\n";
+        }
         read IDFILE, $buffer, 1;
         one_byte(unpack("C",getc(IDFILE)));
         one_byte(unpack("C",getc(IDFILE)));
@@ -619,42 +738,57 @@ my $portion;
 my $past_idx_offset;
 my $iff_size;
 
-my ($sec,$min,$hour,$mday,$month,$year,$wday,$yday,$isdst) = localtime(time);
-
-my $blorbdate = sprintf("%02d%02d%02d at %02d:%02d.%02d",
-                 $year, $month + 1, $mday, $hour, $min, $sec);
-
-print STDOUT "! $version [executing on $blorbdate]\n";
-print STDOUT "! The blorb spell (safely protect a small object ";
-print STDOUT "as though in a strong box).\n";
-
+my $dt = DateTime->now;
+my $blorbdate = sprintf("%s %s %s %s at %s",
+	$dt->day_name, $dt->day, $dt->month_name, $dt->year, $dt->hms);
 
 pod2usage(1) if $options{usage};
 pod2usage(-verbose => 3) if $options{help};
 
+if ($options{inform} && $options{zil}) {
+    print STDERR "** Cannot write more than one manifest at once.\n";
+    pod2usage(1);
+}
+
 if ($ARGV[0]) {
-	$blurb_filename = $ARGV[0];
+    $blurb_filename = $ARGV[0];
 }
 
-if ($ARGV[1]) {
-	$output_filename = "$ARGV[1]";
-}
+if (!$options{inform} && !$options{zil}) { $create_blorb = 1; }
 
-author_chunk("$version on $blorbdate");
-
-if ($options{nobuild})
-{
-    if ($^O eq 'MSWin32' || $^O eq 'MSWin64')
-    {
-        $output_filename = "nul";
-    } else {
-	$output_filename = "/dev/null";
+if ($options{output}) {
+    $output_filename = $options{output};
+} else {
+    if ($create_blorb) {
+	$output_filename = $output_blorb;
     }
 }
 
+if ($output_filename) {
+    open (OUTFILE, ">$output_filename");
+} else {
+    $output_filename = 0;
+    open (OUTFILE, ">&STDOUT");
+}
+
+print_banner "Reading from $blurb_filename";
+ "Writing to $output_filename\n";
+
+print_banner("$version [executing on $blorbdate]");
+print_banner("The blorb spell (safely protect a small object as though in a strong box)");
+if ($options{inform}) {
+    print_banner("Creating Inform6 manifest from $blurb_filename");
+} elsif ($options{zil}) {
+    print_banner("Creating ZIL manifest from $blurb_filename");
+} else {
+    print_banner("Building Blorb file from $blurb_filename");
+}
+
 open (BLURB, $blurb_filename)
     or fatal("can't open blurb file $blurb_filename");
 
+
+author_chunk("$version on $blorbdate");
 while ($c = <BLURB>)
 {
     interpret($c);
@@ -662,6 +796,12 @@ while ($c = <BLURB>)
 
 close BLURB;
 
+# If we're building a manifest, we're done.
+if (!$create_blorb) {
+    print_banner("Manifest complete.");
+    exit 0;
+}
+
 if ($resolution_on == 1)
 {
     begin_chunk("Reso", 0, "");
@@ -712,12 +852,12 @@ open(CHUNK, ">$output_filename")
     or fatal("unable to open $output_filename for output");
 binmode(CHUNK);
 
-print CHUNK "FORM";
+write_chunk("FORM");
 four_word($iff_size - 8);
-print CHUNK "IFRS";
+write_chunk("IFRS");
 
 # Resource Index chunk
-print CHUNK "RIdx";
+write_chunk("RIdx");
 four_word(4 + $important_count*12);
 four_word($important_count);
 
@@ -736,7 +876,7 @@ for ($x = 0; $x < $chunk_count; $x = $x + 1)
 		|| ($type eq "ADRI"))
 	{   $type = "Exec";
 	}
-        print CHUNK $type;
+	write_chunk($type);
         four_word($chunk_number_array[$x]);
         four_word($past_idx_offset + $chunk_offset_array[$x]);
     }
@@ -760,11 +900,11 @@ for ($x = 0; $x < $chunk_count; $x = $x + 1)
         $scount++;
     }
     if ($type ne "AIFF") {
-        print CHUNK $type;
+        write_chunk($type);
         four_word(($chunk_size_array[$x]) - 8);
     }
 
-#print "type: $type  size: ". $chunk_size_array[$x]."\n";
+#print "! type: $type  size: ". $chunk_size_array[$x]."\n";
 
     $chunk_filename = $chunk_filename_array[$x];
     open(CHUNKSUB, $chunk_filename)
@@ -772,21 +912,19 @@ for ($x = 0; $x < $chunk_count; $x = $x + 1)
     binmode(CHUNKSUB);
 
     while(read CHUNKSUB, $portion, 16384) {
-	print CHUNK $portion;
+	write_chunk($portion);
     }
     close(CHUNKSUB);
 
     if (($chunk_size_array[$x] % 2) == 1) {
-	printf CHUNK sprintf("%c", 0);
+	write_chunk(sprintf("%c", 0));
     }
 }
 
 close(CHUNK);
 
-print STDOUT "! Blorb file data written to $output_filename\n";
-print STDOUT "! Completed: size $iff_size bytes ";
-print STDOUT "($pcount pictures, $scount sounds)\n";
-
+print "  Blorb file data written to $output_filename\n";
+print "  Completed: size $iff_size bytes ($pcount pictures, $scount sounds)\n";
 
 # ---------------------------------------------------------------------------
 
@@ -798,7 +936,7 @@ pblorb.pl - Generate a blorb file according to a supplied blurb file
 
 =head1 SYNOPSIS
 
-B<pblorb.pl> - [-n] <story.blurb> [<output.blorb>]
+B<pblorb.pl> [-i | -z] <story.blurb> [-o <output>]
 
 Use -h or --help for verbose help.
 
@@ -811,8 +949,10 @@ B<pblorb.pl> generates a blorb file according to the supplied blurb file.
 =head1 OPTIONS
 	B<-?>            Print simple usage message.
 	B<-h --help>     Print verbose help message.
-	B<-n --nobuild>  Don't build a blorb.  Just parse the blurb file.
-	B< >
+	B<-i --inform>   Emit an Inform6 manifest file.
+	B<-z --zil>      Emit a ZIL manifest file.
+	B<-o --output>   Create a file with this name.
+	S< >
 
 =head1 APPLICATION
 
@@ -821,8 +961,9 @@ executables, sound, graphics, and other resources into a single file for
 use with interactive fiction game interpreters.  The format was
 originally conceived for use with Z-machine and Glulx interpreters, but
 nothing particularly limits it use to these two.  This script also
-provides support for building blorb files for use with ADRIFT and
-Magnetic Scrolls interpreters.
+provides limited support for building blorb files for use with TADS2,
+TADS3, Hugo, Alan, ADRIFT, Level9, AGT, Magnetic Scrolls, and AdvSys
+interpreters.
 
 A blurb file is a text file that describes the contents of the
 soon-to-be-built blorb file.  The blurb is given to pblorb.pl at the
@@ -830,6 +971,15 @@ command line which is then interpreted.  A blorb is then created
 containing the files specified along with any non-file information
 given.
 
+=head1 USAGE
+
+Pblorb is intended to be run twice.  Once to create a manifest file
+prior compilation and again afterwards.  The first pass will create a
+manifest file.  This file provides programmer-friendly constants for
+referring to sound and graphic resources.  Currently manifests for
+Inform6 and ZIL are supported.  Specify which language you want with the
+B<-i> or B<-z> flags.
+
 =head1 GRAMMAR
 
 This section is intended as a quick reference on blurb grammar.  A full
@@ -934,7 +1084,7 @@ Includes this image file as a picture resource marked as "cover art".
 
 =head1 NOTES
 
-This program complies the Blorb Standard version 2.0.4 and the Treaty of
+This program complies with the Blorb Standard version 2.0.4 and the Treaty of
 Babel revision 9.
 
 The Blorb Format was created by Andrew Plotkin in 1998.  For more
@@ -948,6 +1098,6 @@ L<https://en.wikipedia.org/wiki/Interchange_File_Format>
 
 =head1 AUTHORS
     (c) Graham Nelson  1998 (original script to v1.03)
-    (c) David Griffith 2015
+    (c) David Griffith 2012-2019
 
 =cut
