@@ -19,6 +19,10 @@ use Encode qw(decode);
 
 require 5.6.0;
 
+my $name = "scanblorb";
+my $version = "3.1";
+my $tagline = "$name $version";
+
 my $buffer;
 my %options;
 my %images;
@@ -40,13 +44,18 @@ GetOptions('usage|?'	=> \$options{usage},
 	's|sound'	=> \$options{sound},
 	'e|exec'	=> \$options{exec},
 	'x|xml'		=> \$options{xml},
-	'a|all'		=> \$options{all}
+	'a|all'		=> \$options{all},
+	'v|version'	=> \$options{version}
 	);
 
+if ($options{version}) {
+	print "$tagline\n";
+	exit;
+}
+
 my $input_filename = $ARGV[0];
 my $output_filename;
 
-
 pod2usage(1) if $options{usage};
 pod2usage(-verbose => 3) if $options{help};
 pod2usage(1) if !$input_filename;
@@ -62,8 +71,6 @@ if ($options{all}) {
 
 sub array_diff(\@\@);
 
-my $version = "scanBlorb 3.0, part of Blorbtools";
-
 my ($sec,$min,$hour,$mday,$month,$year) = (localtime(time))[0, 1, 2, 3, 4, 5];
 
 my $blorbdate = sprintf("%04d/%02d/%02d at %02d:%02d.%02d",
@@ -85,7 +92,8 @@ my @adrift_magic_500 = (0x3c, 0x42, 0x3f, 0xc9,
 			0x6a, 0x87, 0xc2, 0xcf,
 			0x92, 0x45, 0x3e, 0x61);
 
-print STDOUT "$version [executing on $blorbdate]\n\n";
+print STDOUT "$tagline [executing on $blorbdate]\n";
+print STDOUT "Filename: $input_filename.\n";
 
 open (BLORB, $input_filename) or die "Can't load $input_filename.";
 binmode(BLORB);
@@ -93,12 +101,14 @@ binmode(BLORB);
 read BLORB, $buffer, 12;
 
 my ($groupid, $length, $type) = unpack("NNN", $buffer);
+my $file_length = -s $input_filename;
 
 if (!$groupid || ($groupid != 0x464F524D)) { die "Not a valid FORM file!\n";}
 
 $type == 0x49465253 or die "Not a Blorb file!\n";
 
-print "File length is apparently $length bytes\n";
+print "File length: $file_length bytes\n";
+print "Form length: $length bytes\n";
 
 my ($size, $pos);
 
@@ -114,6 +124,9 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 
 	read(BLORB, $chunkdata, $size) == $size
 		or die("Incomplete chunk at $pos\n");
+
+	if (!$chunkdata) { next; }
+
 	if($size % 2) { read(BLORB, $buffer, 1); }
 
 	# optional chunks
@@ -131,13 +144,26 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 	if ($type eq "ZCOD") {
 		my ($version, $release) = (unpack("CCn", $chunkdata))[0,2];
 		my $serialcode = substr($chunkdata, 0x12, 6);
-	    print "\t$release.$serialcode (version $version)\n";
+		my $informversion = substr($chunkdata, 0x3c, 4);
+		print "\tZ-machine version: $version\n";
+		print "\tCompiler version: $informversion\n";
+		print "\tRelease: $release\n";
+		print "\tSerial number: $serialcode\n";
 	}
 
 	# glulx executable: look into its magic insides
 	if($type eq "GLUL") {
 		my ($major, $minor, $minimus) = (unpack("xxxxnCC", $chunkdata))[0,1,2];
-		print "\tGlulx version $major.$minor.$minimus\n";
+		print "\tGlulx version: $major.$minor.$minimus\n";
+		my $info = substr($chunkdata, 0x24, 4);
+		my $informversion = substr($chunkdata, 0x2c, 4);
+		my $serialcode = substr($chunkdata, 0x36, 6);
+		my $compversion = substr($chunkdata, 0x30, 4);
+		my $release = (unpack("x[53]C", $chunkdata));
+		print "\tCompiler info: $info\n";
+		print "\tCompiler version: $informversion ($compversion)\n";
+		print "\tRelease: $release\n";
+		print "\tSerial number $serialcode\n";
 	}
 
 	# adrift executable: look into its magic insides
@@ -374,7 +400,7 @@ started off as standalone files may be extracted to the currect
 directory by using the appropriate option.  The -a option will cause all
 embedded files to be extracted.
 
-Version 3.0
+Version 3.1
 
 =head1 NOTES
 
